#!/usr/bin/env python
# coding: utf-8

usage = '''
%prog [options] <team-name>
'''

import os
import sys
import json
from optparse import OptionParser
from datetime import datetime

from lpltk import LaunchpadService
from lpltk.LaunchpadService import LaunchpadServiceError
from lpltk.debug import dbg
from lpltk.utils import o2str

if __name__ == "__main__":
    time_started = datetime.now()

    parser = OptionParser(usage=usage)
    parser.add_option('-d', '--debug',
        action='store_true', dest='DEBUG', default=False,
        help='Enable debugging output')
    parser.add_option('-s', '--series',
        action='store', dest='series', default='devel',
        help='Release series name (e.g. sid or lucid or experimental; also devel or stable)')
    parser.add_option('-D', '--distro',
        action='store', dest='distro', default='ubuntu',
        help='Distribution name (e.g. debian or ubuntu)')
    (opt, args) = parser.parse_args()

    if len(args) < 1:
        parser.print_help()
        sys.exit(1)
    if opt.DEBUG:
        os.environ['LPLTK_DEBUG'] = '1'

    try:
        lp = LaunchpadService(config={'read_only': False})
    except LaunchpadServiceError:
        die('Could not connect to Launchpad')

    distro = lp.distributions[opt.distro]
    if opt.series == 'devel' or opt.series == 'unstable':
        series = distro.current_series
    elif opt.series == 'testing':
        series = distro.testing_series
    elif opt.series == 'stable':
        series = distro.stable_series
    else:
        series = distro.get_series(opt.series)
    team_name = args[0]
    team = lp.get_person(team_name)

    series_name = o2str(series.display_name)
    # Hack to fix 'Debian Experimental' series name in Launchpad
    series_name.replace('Debian ', '')
    packages = {
        '_header': {
            'team': o2str(team_name),
            'distro': o2str(distro.display_name),
            'series': series_name,
            'time_started': o2str(time_started),
            }
        }

    dbg("# %s subscribed packages in %s %s" %(
        team_name, distro.display_name, series.display_name))

    for pkg in team.subscribed_package_names:
        packages[pkg] = {}
        for pub in series.get_source_publications(pkg):
            packages[pkg][pub.pocket] = pub.to_dict()

    packages['_header']['time_ended'] = o2str(datetime.now())
    print json.dumps(packages, indent=4)
